/***************************************************************************
                          ksig.cpp  -  description
                             -------------------
    begin                : Tue Jul  9 23:14:22 EDT 2002
    copyright            : (C) 2002 by Scott Wheeler
    email                : wheeler@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ksig.h"
#include "sigmodel.h"

#include <kstandardaction.h>
#include <kactioncollection.h>
#include <kaction.h>
#include <kglobalsettings.h>
#include <klineedit.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <ktextedit.h>
#include <klocale.h>
#include <kinputdialog.h>

#include <qitemselectionmodel.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qsortfilterproxymodel.h>
#include <qsplitter.h>
#include <qtreeview.h>

#include <settings.h>

////////////////////////////////////////////////////////////////////////////////
// public methods
////////////////////////////////////////////////////////////////////////////////

KSig::KSig(QWidget *parent) : KXmlGuiWindow(parent)
    , changed(false)
{
    setupActions();
    setupLayout();
    setupSearchLine();

    if (!initialGeometrySet())
        resize(640,480);
    setupGUI(ToolBar | Keys | StatusBar | Create);
    setAutoSaveSettings();

    readConfig();
    loadData();
}

KSig::~KSig()
{

}

////////////////////////////////////////////////////////////////////////////////
// private methods
////////////////////////////////////////////////////////////////////////////////

void KSig::setupActions()
{
    KStandardAction::openNew(this, SLOT(add()), actionCollection());
    QAction *a;
    a = actionCollection()->addAction( "remove" );
    a->setText(i18n("Remove"));
    a->setIcon(KIcon("edit-delete"));
    connect(a, SIGNAL(triggered()), this, SLOT(remove()));
    KStandardAction::save(this, SLOT(save()), actionCollection());
    KStandardAction::quit(this, SLOT(quit()), actionCollection());

    KStandardAction::cut(actionCollection());
    KStandardAction::copy(actionCollection());
    KStandardAction::paste(actionCollection());
    KStandardAction::clear(actionCollection());

    a = actionCollection()->addAction( "editHeader" );

    a->setText(i18n("Edit Standard Header"));
    connect(a, SIGNAL(triggered()), this, SLOT(editHeader()));
    a = actionCollection()->addAction( "editFooter" );

    a->setText(i18n("Edit Standard Footer"));
    connect(a, SIGNAL(triggered()), this, SLOT(editFooter()));
}

void KSig::setupLayout()
{
    splitter = new QSplitter(Qt::Vertical, this);

    setCentralWidget(splitter);

    m_treeView = new QTreeView(splitter);
    m_treeView->setRootIsDecorated(false);
    m_treeView->setMouseTracking(true);
    m_model = new SigModel(m_treeView);
    m_treeView->setModel(m_model);

    sigEdit = new KTextEdit(splitter );
	sigEdit->setObjectName("sigEdit");
    //sigEdit->setTextFormat(Qt::PlainText);
    sigEdit->setFont(KGlobalSettings::fixedFont());
    sigEdit->setEnabled(false);
    sigEdit->setCheckSpellingEnabled(true);

    statusBar()->insertPermanentItem(i18n(" Line: %1 ", 0), LineNumber, 0);
    statusBar()->insertPermanentItem(i18n(" Col: %1 ", 0), ColumnNumber, 0);
    statusBar()->show();

    updateListLock = false;
    connect(sigEdit, SIGNAL(textChanged()), this, SLOT(updateList()));
    connect(m_treeView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
            this, SLOT(updateEdit()));
    connect(sigEdit, SIGNAL(cursorPositionChanged()), this, SLOT(updateStatus()));
}

void KSig::setupSearchLine()
{
    QSortFilterProxyModel *proxy = new QSortFilterProxyModel(m_treeView);
    proxy->setFilterRole(SigModel::SignatureRole);
    proxy->setFilterCaseSensitivity(Qt::CaseInsensitive);
    proxy->setSourceModel(m_model);
    m_treeView->setModel(proxy);
    connect(m_treeView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
            this, SLOT(updateEdit()));

    QWidget *searchWidget = new QWidget();
    QHBoxLayout *lay = new QHBoxLayout(searchWidget);
    lay->setMargin(0);
    QLabel *label = new QLabel(i18n("Search:"), searchWidget);
    lay->addWidget(label);
    KLineEdit *lineEdit = new KLineEdit(searchWidget);
    lineEdit->setClearButtonShown(true);
    lay->addWidget(lineEdit);
    connect(lineEdit, SIGNAL(textChanged(QString)), proxy, SLOT(setFilterFixedString(QString)));

    KAction *act = actionCollection()->addAction( "search" );
    act->setText( i18n("Search") );
    act->setDefaultWidget(searchWidget);

}

void KSig::loadData()
{
    if(m_treeView->model()->rowCount() > 0) {
        m_treeView->setCurrentIndex(m_treeView->model()->index(0, 0));

        sigEdit->setEnabled(true);
    }
    else {
        updateListLock = true;
        sigEdit->setText(i18n("To get started, first create a new signature by selecting "
                              "\"New\" above. You will then be able to edit and save "
                              "your collection of signatures."));
        updateListLock = false;
    }
    setDataChanged(false);
}

void KSig::readConfig()
{
    setAutoSaveSettings();

    QList<int> sizes;
    sizes.append(Settings::listSize());
    sizes.append(Settings::editSize());
    if(sizes.at(0) > 0 && sizes.at(1) > 0)
	splitter->setSizes(sizes);
}

void KSig::writeConfig()
{
    const QList<int> sizes = splitter->sizes();
    Settings::setListSize(sizes.at(0));
    Settings::setEditSize(sizes.at(1));

    Settings::self()->writeConfig();
}

bool KSig::queryClose()
{
    if(changed) {
	int saveChanges = KMessageBox::questionYesNoCancel(this, i18n("Do you want to save your changes before exiting?"), QString(), KStandardGuiItem::save(), KStandardGuiItem::discard());
	if(saveChanges == KMessageBox::Cancel)
	    return(false);
	else if(saveChanges == KMessageBox::Yes)
	    save();
    }

    writeConfig();
    return(true);
}

////////////////////////////////////////////////////////////////////////////////
// private slots
////////////////////////////////////////////////////////////////////////////////

void KSig::add()
{
    setDataChanged();
    const QModelIndex index = m_treeView->currentIndex();
    const int baseRow = index.isValid() ? index.row() : 0;
    m_treeView->model()->insertRow(baseRow);
    m_treeView->setCurrentIndex(m_treeView->model()->index(baseRow, 0));

    sigEdit->setEnabled(true);
    sigEdit->setFocus();
}

void KSig::remove()
{
    const QModelIndex index = m_treeView->currentIndex();
    if(!index.isValid())
	return;

    setDataChanged();
    m_treeView->model()->removeRow(index.row());
}

void KSig::save()
{
    m_model->save();
    setDataChanged(false);
}

void KSig::setDataChanged(bool value)
{
    changed = value;
    setCaption(QString::null, changed);	//krazy:exclude=nullstrassign for old broken gcc
}

void KSig::editHeader()
{
    bool ok = false;
    const QString caption = i18n("Edit Header");
    const QString label = i18n("Standard signature header:");
    const QString text = KInputDialog::getMultiLineText(caption, label, Settings::header(), &ok, this);
    if(ok)
	Settings::setHeader(text);
}

void KSig::editFooter()
{
    bool ok = false;
    const QString caption = i18n("Edit Footer");
    const QString label = i18n("Standard signature footer:");
    const QString text = KInputDialog::getMultiLineText(caption, label, Settings::footer(), &ok, this);
    if(ok)
	Settings::setFooter(text);
}

void KSig::updateList()
{
    if(!updateListLock) {
	setDataChanged();
	const QModelIndex index = m_treeView->currentIndex();
	if(index.isValid())
	    m_treeView->model()->setData(index, sigEdit->toPlainText());
    }
}

void KSig::updateEdit()
{
    updateListLock = true;

    const QModelIndex index = m_treeView->currentIndex();
    if(index.isValid())
	sigEdit->setText(index.data(SigModel::SignatureRole).toString());
    else {
	sigEdit->clear();
	sigEdit->setEnabled(false);
    }

    updateListLock = false;
}

void KSig::updateStatus()
{
    const QTextCursor cursor = sigEdit->textCursor();
    const int line = cursor.blockNumber();
    const int column = cursor.columnNumber();
    statusBar()->changeItem(i18n(" Line: %1 ", line + 1), LineNumber);
    statusBar()->changeItem(i18n(" Col: %1 ", column + 1), ColumnNumber);
}

void KSig::quit()
{
    close();
}

#include "ksig.moc"
