/***************************************************************************
 *   Copyright (C) 2002 by Scott Wheeler <wheeler@kde.org>                 *
 *   Copyright (C) 2012 by Pino Toscano <pino@kde.org>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "sigmodel.h"
#include "sigmodel.moc"

#include <qregexp.h>
#include <qtextstream.h>

#include <kglobal.h>
#include <klocale.h>
#include <kstandarddirs.h>

static void nodeToText(const QDomNode &n, QString &s)
{
    QDomNodeList children = n.childNodes();

    for(int i = 0; i < children.count(); i++) {
	if(children.item(i).isText())
	    s.append(children.item(i).toText().data());
	else {
	    nodeToText(children.item(i), s);
	    if(children.item(i).isElement() && children.item(i).toElement().tagName() == "p") {
		s.append("\n");
	    }
	}
    }
}

static QString signatureString(const QDomNode &n)
{
    QString text;
    nodeToText(n, text);
    text.replace(QRegExp("\n$"), "");
    return text;
}


SigModel::SigModel(QObject *parent) : QAbstractItemModel(parent)
{
    QString dir = KGlobal::dirs()->saveLocation("appdata");
    if(!dir.isNull())
	load(dir + "sigs.sigml");
}

SigModel::~SigModel()
{
}

int SigModel::columnCount(const QModelIndex &parent) const
{
    return parent.isValid() ? 0 : 1;
}

QVariant SigModel::data(const QModelIndex &index, int role) const
{
    if(!index.isValid() || index.column() != 0 || index.row() < 0 || index.row() >= m_signatures.count())
	return QVariant();

    switch(role) {
    case Qt::DisplayRole: {
	const QString sig = m_cachedSignatures.at(index.row());
	return sig.isEmpty() ? emptySignatureString() : sig.simplified();
    }
    case SignatureRole:
	return m_cachedSignatures.at(index.row());
    }
    return QVariant();
}

QVariant SigModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if(section != 0 || orientation != Qt::Horizontal)
	return QVariant();

    switch(role) {
    case Qt::DisplayRole:
	return i18n("Signatures");
    }
    return QVariant();
}

QModelIndex SigModel::index(int row, int column, const QModelIndex &parent) const
{
    if(column != 0 || row < 0 || row >= m_signatures.count() || parent.isValid())
	return QModelIndex();

    return createIndex(row, column);
}

bool SigModel::insertRows(int row, int count, const QModelIndex &parent)
{
    if(parent.isValid() || row < 0 || row > m_signatures.count())
	return false;

    beginInsertRows(parent, row, row + count - 1);
    QDomNode node;
    // insert the first row distinguish whether it is in the middle
    // or at the end, and whether there are no elements
    if(m_signatures.isEmpty()) {
	QDomElement newElement = m_doc.createElement("signature");
	node = m_rootElement.appendChild(newElement);
    } else if(row < m_signatures.count()) {
	QDomElement newElement = m_doc.createElement("signature");
	node = m_signatures.at(row);
	node = node.parentNode().insertBefore(newElement, node);
    } else {
	QDomElement newElement = m_doc.createElement("signature");
	node = m_signatures.at(m_signatures.count() - 1);
	node = node.parentNode().insertAfter(newElement, node);
    }
    // start from 1, as above one node has been added already
    for(int i = 1; i < count; ++i) {
	QDomElement newElement = m_doc.createElement("signature");
	node = node.parentNode().insertAfter(newElement, node);
    }
    m_signatures = m_doc.elementsByTagName("signature");
    m_cachedSignatures.insert(row, count, QString());
    endInsertRows();
    return true;
}

QModelIndex SigModel::parent(const QModelIndex &) const
{
    return QModelIndex();
}

bool SigModel::removeRows(int row, int count, const QModelIndex &parent)
{
    if(parent.isValid() || row < 0 || row >= m_signatures.count())
	return false;

    beginRemoveRows(parent, row, row + count - 1);
    for(int i = 0; i < count; ++i)
	m_signatures.at(row).parentNode().removeChild(m_signatures.at(row));
    m_signatures = m_doc.elementsByTagName("signature");
    m_cachedSignatures.remove(row, count);
    endRemoveRows();
    return true;
}

int SigModel::rowCount(const QModelIndex &parent) const
{
    return parent.isValid() ? 0 : m_signatures.count();
}

bool SigModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if(!index.isValid() || index.column() != 0 || index.row() < 0 || index.row() >= m_signatures.count())
	return false;

    switch(role) {
    case Qt::EditRole:
	m_cachedSignatures[index.row()] = value.toString();
	emit dataChanged(index, index);
	return true;
    }

    return false;
}

void SigModel::save()
{
   if(m_file.fileName().isEmpty())
	return;

    for(int i = 0; i < m_signatures.count(); ++i) {
	updateNode(m_signatures.at(i), m_cachedSignatures.at(i));
    }

    if(m_file.open(QIODevice::WriteOnly)) {
	QTextStream stream(&m_file);
	stream << m_doc;
	m_file.close();
    }
}

QString SigModel::emptySignatureString() const
{
    if(m_emptySigString.isEmpty())
	m_emptySigString = i18n("<empty signature>");
    return m_emptySigString;
}

void SigModel::load(const QString &fileName)
{
    m_file.setFileName(fileName);

    if(m_file.open(QIODevice::ReadOnly) && m_doc.setContent(&m_file)) {

	// find the root element
	QDomNodeList topLevelElements = m_doc.childNodes();
	int i = 0;
	while(topLevelElements.item(i).toElement().tagName() != "SigML" && i < topLevelElements.count())
	    i++;

	if(i < topLevelElements.count())
	    // if we didn't hit the end of the list
	    m_rootElement = topLevelElements.item(i).toElement();
	else {
	    // if we didn't find the root element, create one
	    m_rootElement = m_doc.createElement("SigML");
	    m_doc.appendChild(m_rootElement);
	}

	m_signatures = m_doc.elementsByTagName("signature");
	m_cachedSignatures.resize(m_signatures.count());
	for(int i = 0; i < m_signatures.count(); ++i)
	    m_cachedSignatures[i] = signatureString(m_signatures.at(i));
    }
    // if the document could not be opened or setData failed, create the document framework
    else {
	m_rootElement = m_doc.createElement("SigML");
	m_doc.appendChild(m_rootElement);
    }
    m_file.close();
}

void SigModel::updateNode(QDomNode element, const QString &text)
{
    while(!element.firstChild().isNull())
	element.removeChild(element.firstChild());

    // create new children
    const QStringList lines = text.split('\n', QString::KeepEmptyParts);

    foreach(const QString &line, lines) {
	QDomElement p = m_doc.createElement("p");
	element.appendChild(p);
	p.appendChild(m_doc.createTextNode(line));
    }
}

